# -*- shell-script -*-
# git-package completion

_git_package()
{
    local cur prev words cword
    if [ "$(type -t _init_comletion)" != '' ]
    then
	_init_completion || return
    else
	cword=$COMP_CWORD
	cur=${COMP_WORDS[cword]}
	prev=${COMP_WORDS[cword-1]}
    fi

    BUILD_DISTS=$(git config --get-all package.distribution)
    if [ -z "${BUILD_DISTS}" ]
    then
	BUILD_DISTS="eole-2.4/experimental eole-2.4/unstable envole-4/experimental envole-4/unstable"
    fi

    declare -A HELP=( ["-h"]="--help" ["-v"]="--version"
        ["-c"]="--copyright" ["-l"]="--licence" ["X-inval"]="--changes" )
    declare -A OPTIONS=( ["-b"]="--build" ["-s"]="--sign" ["-d"]="--distrib" )

    case "${prev}" in
        git-package|package)
            COMPREPLY=( $(_git_package_options "--build ${HELP[*]}") )
            ;;
        -b|--build)
            COMPREPLY=( $(_git_package_options "--distrib --sign ${HELP[*]}") )
            ;;
        -d|--distrib)
            COMPREPLY=( $(_git_package_options "--sign ${HELP[*]}" "${BUILD_DISTS}") )
            ;;
        -s|--sign)
            COMPREPLY=( $(_git_package_options "--key ${HELP[*]}") )
            ;;
        -k|--key)
            COMPREPLY=( $( compgen -W "$( gpg2 --list-secret-keys 2>/dev/null | \
                sed -ne 's@^.*<\([^>]*\)>.*$@\1@p')" -- "${cur}" ) )
            ;;
        *)
            COMPREPLY=( $(_git_package_options) )
            ;;
    esac
    # Add extra space after option
    # Disable filename completion
    compopt +o nospace +o default
    return 0
} &&
complete -F _git_package git-package

_git_package_options()
{
    local temp="${1}"
    [ -z "${temp}" ] && temp="${OPTIONS[@]} ${HELP[@]}"

    if ! git show-ref -q 2> /dev/null
    then
	temp="${HELP[@]}"
	# Display only help when outside git repository
	compgen -W "${temp}" -- "${cur}"
	return 0
    fi

    # strip already passed options
    for i in $(seq 0 $((cword-1)))
    do
        # Short or long option?
        if [ -n "${HELP[${COMP_WORDS[$i]}]}" ] \
            || [ "${HELP[*]}" != "${HELP[*]/${COMP_WORDS[$i]}}" ]
        then
            # Useless to pass options with any help
            return 0
        elif [ -n "${OPTIONS[${COMP_WORDS[$i]}]}" ]
        then
            # Short option
            temp="${temp//${OPTIONS[${COMP_WORDS[$i]}]}}"
        elif [ "${OPTIONS[*]}" != "${OPTIONS[*]//${COMP_WORDS[$i]}}" ]
        then
            # Long option
            temp="${temp//${COMP_WORDS[$i]}}"
        fi
    done

    case "$cur" in
        -*)
            compgen -W "${temp}" -- "${cur}"
            ;;
        *)
            if [ -n "${2}" ]
            then
                compgen -W "${2}" -- "${cur}"
            else
                _git_package_find_prefix "${cur}"
            fi
            ;;
    esac
    return 0
}

_git_package_find_prefix()
{
    local name="${1}"
    local remote=
    local prefix=
    local dist_branches=
    local remotes=$(git remote)

    [ -z "$(__gitdir)" ] && COMPREPLY=( '' ) && return 0

    # Get all local and remote heads
    for ref in $(git for-each-ref --format='%(refname)' refs/{heads,remotes}/)
    do
	# Optimize when a name is provided
	[ -z "${name}" ] || [[ "${ref}" =~ "${name}" ]] || continue
	if [[ "${ref}" =~ ^refs/heads/ ]]
	then
	    ref="${ref#refs/heads/}"
	elif [[ "${ref}" =~ ^refs/remotes/ ]]
	then
	    ref="${ref#refs/remotes/}"
	    for remote_name in $remotes
	    do
		if [[ "${ref}" =~ ^$remote_name ]]
		then
		    # Strip remote name from ref
		    # but keep it for “git cat-file”
		    ref="${ref#$remote_name/}"
		    remote="${remote_name}/"
		    break
		fi
	    done
	fi
	if git cat-file -e "${remote}${ref}:debian/control" 2> /dev/null
	then
	    # One branch per line to remove duplicates
	    # Avoid empty blank lines
	    dist_branches="${dist_branches:+${dist_branches}$'\n'}${ref}"
	fi
    done
    # Avoid duplicates
    dist_branches=$(printf "%s" "${dist_branches}" | sort | uniq)

    if [ -n "${dist_branches}" ]
    then
        for branch in ${dist_branches}
        do
            [ -z "$prefix" ] && prefix="$branch" && continue
            prefix=$(_git_find_longest "$prefix" "$branch")
        done
        compgen -W "${dist_branches}" -- "${prefix}"
    fi
}

_git_find_longest()
{
    s="$1"
    t="$2"
    l=1

    while [ "${t#${s:0:$l}}" != "$t" ]
    do
        (( l = l + 1 ))
    done
    (( l = l - 1 ))

    printf "%s" "${s:0:$l}"
}

# ex: ts=4 sw=4 et filetype=sh
